#region Copyright

/*
 * Copyright 2005-2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

#endregion

using System;
#if NET_1_1
// NET 1.1
using System.Collections;
#else
// NET 2.0
using System.Collections.Generic;
#endif
using System.IO;
using System.Reflection;
using log4net;
using log4net.Config;
using log4net.Util;
using MbUnit.Framework;
#if NET_1_1
// NET 1.1
using Nullables;
#endif
using Seasar.Extension.Unit;
using Seasar.WindowsExample.Logics.Dao;
using Seasar.WindowsExample.Logics.Dto;
using Seasar.WindowsExample.Logics.Service;

namespace Seasar.WindowsExample.Tests
{
    /// <summary>
    /// ЈpeXgP[XNX
    /// </summary>
    [TestFixture]
    public class TestEmployee : S2TestCase
    {
        /// <summary>
        /// Logicݒt@C
        /// </summary>
        private const string PATH = "ExampleLogics.dicon";

        /// <summary>
        /// eXg̃ZbgAbv
        /// </summary>
        [SetUp]
        public void Setup()
        {
            FileInfo info = new FileInfo(
                string.Format("{0}.dll.config", SystemInfo.AssemblyShortName(
                                                    Assembly.GetExecutingAssembly())));
            // AZudll̏ꍇ".dll.config"

            XmlConfigurator.Configure(LogManager.GetRepository(), info);
        }

        /// <summary>
        /// ñeXg
        /// </summary>
        [Test, S2]
        public void TestSelectOfDao()
        {
            Include(PATH);

            IEmployeeDao dao = (IEmployeeDao) GetComponent(typeof (IEmployeeDao));
            Assert.IsNotNull(dao, "NotNull");

            // ꗗŎ擾
#if NET_1_1
            // NET 1.1
            
            IList list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#else
            // NET 2.0
            
            IList<EmployeeDto> list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#endif
            int i = 0;
            foreach (EmployeeDto dto in list)
            {
                if ( i == 2 )
                {
                    Assert.AreEqual(3, dto.Id.Value, "Id");
                    Assert.AreEqual("q", dto.Name, "Name");
                    Assert.AreEqual("010003", dto.Code, "Code");
                    Assert.AreEqual(2, dto.Gender, "Gender");
                    Assert.AreEqual(new DateTime(2001, 4, 1, 0, 0, 0),
                                    dto.EntryDay.Value, "Entry");
                    Assert.AreEqual(1, dto.DeptNo.Value, "DeptNo");
                    Assert.AreEqual(1, dto.Department.Id.Value, "Dept.No");
                    Assert.AreEqual("cƕ", dto.Department.Name, "Dept.Name");
                    Assert.AreEqual("0001", dto.Department.Code, "Dept.Code");
                }
                i++;
            }

            // ʂɎ擾

            EmployeeDto data = dao.GetData(3);

            Assert.AreEqual(3, data.Id.Value, "Id2");
            Assert.AreEqual("q", data.Name, "Name2");
            Assert.AreEqual("010003", data.Code, "Code2");
            Assert.AreEqual(2, data.Gender, "Gender2");
            Assert.AreEqual(new DateTime(2001, 4, 1, 0, 0, 0),
                            data.EntryDay.Value, "Entry2");
            Assert.AreEqual(1, data.DeptNo.Value, "DeptN2o");
            Assert.AreEqual(1, data.Department.Id.Value, "Dept.No2");
            Assert.AreEqual("cƕ", data.Department.Name, "Dept.Name2");
            Assert.AreEqual("0001", data.Department.Code, "Dept.Code2");

            Assert.AreEqual(3, dao.GetId("010003"), "GetId");
        }

        /// <summary>
        /// XVñeXg
        /// </summary>
        [Test, S2(Tx.Rollback)]
        public void TestInsertOfDao()
        {
            Include(PATH);

            IEmployeeDao dao = (IEmployeeDao) GetComponent(typeof (IEmployeeDao));
            Assert.IsNotNull(dao, "NotNull");

            // }̃eXg
            EmployeeDto data = new EmployeeDto();
#if NET_1_1
            // NET 1.1
            data.Code = "060006";
            data.Name = "㓡ZY";
            data.EntryDay = new NullableDateTime(new DateTime(2006, 4, 1, 0, 0, 0));
            data.Gender = 1;
            data.DeptNo = 1;
#else
            // NET 2.0
            data.Code = "060006";
            data.Name = "㓡ZY";
            data.EntryDay = new DateTime(2006, 4, 1, 0, 0, 0);
            data.Gender = 1;
            data.DeptNo = 1;
#endif

            Assert.AreEqual(1, dao.InsertData(data), "Insert");

            // XṼeXg
            int id = dao.GetId("060006");
#if NET_1_1
            // NET 1.1
            data.Id = id;
            data.Code = "060006";
            data.Name = "ܓZY";
            data.EntryDay = new NullableDateTime(new DateTime(2006, 4, 1, 0, 0, 0));
            data.Gender = 1;
            data.DeptNo = 2;
#else
            // NET 2.0
            data.Id = id;
            data.Code = "060006";
            data.Name = "ܓZY";
            data.EntryDay = new DateTime(2006, 4, 1, 0, 0, 0);
            data.Gender = 1;
            data.DeptNo = 2;
#endif

            Assert.AreEqual(1, dao.UpdateData(data), "Update");

            data = dao.GetData(id);
            Assert.AreEqual(id, data.Id.Value, "Id");
            Assert.AreEqual("ܓZY", data.Name, "Name");
            Assert.AreEqual("060006", data.Code, "Code");
            Assert.AreEqual(1, data.Gender, "Gender2");
            Assert.AreEqual(new DateTime(2006, 4, 1, 0, 0, 0),
                            data.EntryDay.Value, "Entry");
            Assert.AreEqual(2, data.DeptNo.Value, "DeptNo");
            Assert.AreEqual(2, data.Department.Id.Value, "Dept.No2");
            Assert.AreEqual("Zp", data.Department.Name, "Dept.Name2");
            Assert.AreEqual("0002", data.Department.Code, "Dept.Code2");

            // 폜̃eXg
            data = new EmployeeDto();
            data.Id = id;

            Assert.AreEqual(1, dao.DeleteData(data), "Delete");
#if NET_1_1
            // NET 1.1
            
            IList list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#else
            // NET 2.0
            
            IList<EmployeeDto> list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#endif
        }

        /// <summary>
        /// CSVpeXg
        /// </summary>
        [Test, S2]
        public void TestDaoOfCSV()
        {
            Include(PATH);

            // 擾̃eXg
            IEmployeeCSVDao dao = (IEmployeeCSVDao) GetComponent(typeof (IEmployeeCSVDao));
            Assert.IsNotNull(dao, "NotNull");

#if NET_1_1
            // NET 1.1
            
            IList list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#else
            // NET 2.0
            
            IList<EmployeeCsvDto> list = dao.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#endif
            int i = 0;
            foreach (EmployeeCsvDto dto in list)
            {
                if ( i == 2 )
                {
                    Assert.AreEqual("q", dto.Name, "Name");
                    Assert.AreEqual("010003", dto.Code, "Code");
                    Assert.AreEqual(2, dto.Gender, "Gender");
                    Assert.AreEqual("", dto.GenderName, "GenderName");
                    Assert.AreEqual(new DateTime(2001, 4, 1, 0, 0, 0),
                                    dto.EntryDay.Value, "Entry");
                    Assert.AreEqual("cƕ", dto.DeptName, "Dept.Name");
                    Assert.AreEqual("0001", dto.DeptCode, "Dept.Code");
                }
                i++;
            }

            // o͂̃eXg
            string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\csvtest.csv";
            IOutputCSVDao daoOfCsv = (IOutputCSVDao) GetComponent(typeof (IOutputCSVDao));
            Assert.AreEqual(5, daoOfCsv.OutputEmployeeList(path, list));
        }

        /// <summary>
        /// ЈXgT[rXeXg
        /// </summary>
        [Test, S2]
        public void TestListService()
        {
            Include(PATH);

            IEmployeeListService service = (IEmployeeListService) GetComponent(typeof (IEmployeeListService));
            Assert.IsNotNull(service, "NotNull");

#if NET_1_1
            // NET 1.1
            
            IList list = service.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#else
            // NET 2.0
            
            IList<EmployeeDto> list = service.GetAll();
            Assert.AreEqual(5, list.Count, "Count");
#endif
            int i = 0;
            foreach (EmployeeDto dto in list)
            {
                if ( i == 2 )
                {
                    Assert.AreEqual(3, dto.Id.Value, "Id");
                    Assert.AreEqual("q", dto.Name, "Name");
                    Assert.AreEqual("010003", dto.Code, "Code");
                    Assert.AreEqual(2, dto.Gender, "Gender");
                    Assert.AreEqual(new DateTime(2001, 4, 1, 0, 0, 0),
                                    dto.EntryDay.Value, "Entry");
                    Assert.AreEqual(1, dto.DeptNo.Value, "DeptNo");
                    Assert.AreEqual(1, dto.Department.Id.Value, "Dept.No");
                    Assert.AreEqual("cƕ", dto.Department.Name, "Dept.Name");
                    Assert.AreEqual("0001", dto.Department.Code, "Dept.Code");
                    Assert.AreEqual("cƕ", dto.DeptName, "DeptName");
                }
                i++;
            }
        }

        /// <summary>
        /// Јo^T[rX̃eXg
        /// </summary>
        [Test, S2(Tx.Rollback)]
        public void TestEditService()
        {
            Include(PATH);

            IEmployeeEditService service = (IEmployeeEditService) GetComponent(typeof (IEmployeeEditService));
            Assert.IsNotNull(service, "NotNull");

            EmployeeDto data = service.GetData(3);
            Assert.AreEqual(3, data.Id.Value, "Id");
            Assert.AreEqual("q", data.Name, "Name");
            Assert.AreEqual("010003", data.Code, "Code");
            Assert.AreEqual(2, data.Gender, "Gender");
            Assert.AreEqual(new DateTime(2001, 4, 1, 0, 0, 0),
                            data.EntryDay.Value, "Entry");
            Assert.AreEqual(1, data.DeptNo.Value, "DeptNo");
            Assert.AreEqual(1, data.Department.Id.Value, "Dept.No");
            Assert.AreEqual("cƕ", data.Department.Name, "Dept.Name");
            Assert.AreEqual("0001", data.Department.Code, "Dept.Code");
            Assert.AreEqual("cƕ", data.DeptName, "DeptName");

            // }̃eXg
            data = new EmployeeDto();
#if NET_1_1
            // NET 1.1
            data.Code = "060006";
            data.Name = "㓡ZY";
            data.EntryDay = new NullableDateTime(new DateTime(2006, 4, 1, 0, 0, 0));
            data.Gender = 1;
            data.DeptNo = 1;
#else
            // NET 2.0
            
            data.Code = "060006";
            data.Name = "㓡ZY";
            data.EntryDay = new DateTime(2006, 4, 1, 0, 0, 0);
            data.Gender = 1;
            data.DeptNo = 1;
#endif

            Assert.AreEqual(1, service.ExecUpdate(data), "Insert");

            // XṼeXg
            data = new EmployeeDto();
#if NET_1_1
            // NET 1.1
            data.Id = 2;
            data.Code = "999999";
            data.Name = "ؓY";
            data.EntryDay = new NullableDateTime(new DateTime(1999, 5, 1, 0, 0, 0));
            data.Gender = 1;
            data.DeptNo = 2;
#else
            // NET 2.0
            
            data.Id = 2;
            data.Code = "999999";
            data.Name = "ؓY";
            data.EntryDay = new DateTime(1999, 5, 1, 0, 0, 0);
            data.Gender = 1;
            data.DeptNo = 2;
#endif

            Assert.AreEqual(1, service.ExecUpdate(data), "Update");

            data = service.GetData(2);
            Assert.AreEqual(2, data.Id.Value, "Id");
            Assert.AreEqual("ؓY", data.Name, "Name");
            Assert.AreEqual("999999", data.Code, "Code");
            Assert.AreEqual(1, data.Gender, "Gender");
            Assert.AreEqual(new DateTime(1999, 5, 1, 0, 0, 0),
                            data.EntryDay.Value, "Entry");
            Assert.AreEqual(2, data.DeptNo.Value, "DeptNo");
            Assert.AreEqual(2, data.Department.Id.Value, "Dept.No");
            Assert.AreEqual("Zp", data.Department.Name, "Dept.Name");
            Assert.AreEqual("0002", data.Department.Code, "Dept.Code");
            Assert.AreEqual("Zp", data.DeptName, "DeptName");
        }
    }
}